from PyQt5.QtCore import *
from PyQt5.QtWidgets import *
from model.member import Member
from utils.members_table import MembersTable
from windows import groupsoverview
from globals import Global
from utils.groups_table import GroupsTable
from model.group import Group


class GroupsOverview(QWidget, groupsoverview.Ui_Form):
    _BUTTON_WIDTH = 50

    _edit_buttons_members = []
    _edit_buttons_groups = []
    
    def __init__(self, main_window, parent=None):
        super(GroupsOverview, self).__init__(parent)
        self.main_window = main_window
        self.setupUi(self)
        self.btnCreateGroup.clicked.connect(main_window.goToGroupEdit)

        # add groups table
        self.groups_table = GroupsTable(num_of_buttons=2, button_titles=(self.tr("rozdělit"), self.tr("upravit"),), data=Global.db.getGroups())
        self.layoutTables.addWidget(self.groups_table)
        self.groups_table.btnClicked.connect(self.onGroupBtnClicked)
        self.groups_table.itemDoubleClicked.connect(self.editGroup)
        self.groups_table.elementSelected.connect(self.showMembers)

        # add members table
        self.members_table = MembersTable(num_of_buttons=1, button_titles=("view",))
        self.layoutTables.addWidget(self.members_table)
        self.members_table.btnClicked.connect(self.viewMember)
        self.members_table.itemDoubleClicked.connect(self.viewMember)

    # reload data, because some other controller changed them
    def reloadData(self):
        self.groups_table.changeData(Global.db.getGroups())
        self.members_table.changeData([])

    # changes content of members table to correspond with groups table selection
    @pyqtSlot(Group)
    def showMembers(self, group):
        self.members_table.changeData(group.members)

    @pyqtSlot(Group, int)
    def onGroupBtnClicked(self, group, btn_id):
        if btn_id == 0:
            self.distribute(group)
        if btn_id == 1:
            self.editGroup(group)

    @pyqtSlot(Group)
    def editGroup(self, group):
        self.main_window.goToGroupEdit(group)

    @pyqtSlot(Member)
    def viewMember(self, person):
        self.main_window.goToMemberOverview(person)

    def distribute(self, group):
        self.main_window.goToDistributionSetup(group)
