from globals import Global


class Member:
    def __init__(self, id_, name, group_id):
        self.id_ = id_
        self.name = name
        self.group_id = group_id
        self._history_count = None

    @property
    def group_name(self):
        return Global.db.getGroupName(self.group_id)

    @property
    def history_count(self):
        if self._history_count is None:
            self._history_count = Global.db.getHistoryCount(self.id_)
        return self._history_count

    def delete(self):
        """
        delete this member from database completely (also from every distribution he was in)
        """
        Global.db.deleteMember(self.id_)

    def removeFromGroup(self):
        """
        remove this member from current group
        """
        Global.db.unsetGroupFromMember(self.id_)

    def rename(self, new_name):
        """
        rename this member
        :param new_name: 
        :raise database.IntegrityError if the name is not unique in member's group
        """
        if new_name == self.name:
            return
        Global.db.changeMemberName(self.id_, new_name)
        self.name = new_name

    def __eq__(self, other):
        if not isinstance(other, Member):
            return False
        return self.id_ == other.id_

    def __hash__(self):
        return self.id_
