from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QPushButton

from model.requirements import RequirementSameTeam
from utils.qtmodels.groups_table_model import GroupsTableModel
from utils.qtmodels.requirements_table_model import RequirementsTableModel
from utils.table import ElemTable


class RequirementsTable(ElemTable):
    def __init__(self, num_of_buttons=0, button_titles=(), data=[], parent=None):
        super(RequirementsTable, self).__init__(RequirementsTableModel(data, num_of_buttons + 3), num_of_buttons, button_titles, parent)

    def changeData(self, data):
        self.setModel(GroupsTableModel(data, self._btn_amount + 3))

    def insertRequirement(self, requirement):
        self.model.setDynamicSortFilter(False)
        self.model.sourceModel().insertRequirement(0, requirement)

        # add buttons to new row
        for btn_nr in range(self._btn_amount):
            button = QPushButton(self._btn_titles[btn_nr])
            button.clicked.connect(self.onBtnClicked)
            requirement = self.model.data(self.model.index(0, 0), Qt.UserRole)
            if isinstance(requirement, RequirementSameTeam):
                req_type = "same"
            else:
                req_type = "diff"
            button.setObjectName(str(requirement.member) + "_" + req_type + "_" + str(btn_nr))
            self.table.setIndexWidget(self.model.index(0, btn_nr+self._data_columns), button)

        self.model.setDynamicSortFilter(True)

    def onBtnClicked(self):
        req_person, req_type, btn_id = str.split(self.sender().objectName(), '_')
        if req_type == "same":
            is_same_team = True
        elif req_type == "diff":
            is_same_team = False
        requirement = self.model.sourceModel().getRequirement(req_person, is_same_team)
        self.btnClicked.emit(requirement, int(btn_id))

    def getAllRequirements(self):
        return self.source_model.requirements
